# The contents of this file are subject to the Mozilla Public
# License Version 1.1 (the "License"); you may not use this file
# except in compliance with the License. You may obtain a copy of
# the License at http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS
# IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
# implied. See the License for the specific language governing
# rights and limitations under the License.
#
# The Original Code is BesserWisser version 2.0.
#
# The Initial Developer of the Original Code is European Environment
# Agency (EEA).  Portions created by Finsiel Romania are
# Copyright (C) European Environment Agency.  All
# Rights Reserved.
#
# Contributor(s):
# Soren Roug, EEA
# Miruna Badescu, Finsiel Romania


""" Container for all BesserWisser objects:
    * 'Expert Category' - contain 'Expert Question's
    * 'Expert Folder'   - contains experts: 'Expert' objects
"""

# Python imports
import string
import operator

# Zope imports
import Globals
from OFS.Folder import Folder
from AccessControl import getSecurityManager, ClassSecurityInfo, Unauthorized
from AccessControl.Permissions import view_management_screens, view
from Products import meta_types
from DateTime import DateTime


# Product imports
from SessionManager import SessionManager
from CatalogManager import CatalogManager
from LanguagesManager import LanguagesManager
from EmailManager import EmailManager
from Utils import Utils
import Category
import Question
import ExpertsFolder
import Expert
from BWCookieCrumbler import BWCookieCrumbler
import Resources

manage_addBesserWisserForm = Globals.DTMLFile('dtml/BesserWisser_add', globals())
def manage_addBesserWisser(self, id, title='', longtitle='', p_servername='', p_serverport='', adminEmail='', REQUEST=None):
    """ Adds a Besser Wisser object """
    #some conversions
    try:
        p_serverport = int(p_serverport)
    except:
        p_serverport = Resources.BESSER_WISSER_DEFAULT_EMAIL_SERVER_PORT
    ob = BesserWisser(id, title, longtitle, p_servername, p_serverport, adminEmail, self.absolute_url(1) + id)
    self._setObject(id, ob)
    ob=self._getOb(id)
    # Experts folder inside
    obj = ExpertsFolder.manage_addExpertsFolder(ob, title='Experts Folder')
    checkPermission=getSecurityManager().checkPermission
    if not checkPermission('Add User Folders', ob):
        raise Unauthorized, ('You are not authorized to add User Folders.')
    ob.manage_addUserFolder()
    ob.LoadINI()
    if REQUEST is not None:
        return self.manage_main(self, REQUEST, update_menu=1)

class BesserWisser(BWCookieCrumbler,
    Folder,
    Utils,
    LanguagesManager,
    CatalogManager,
    SessionManager,
    EmailManager):
    """ Container for all BesserWisser objects  """

    meta_type = Resources.METATYPE_BESSER_WISSER
    icon = 'misc_/BesserWisser/engine.gif'

    security = ClassSecurityInfo()

    manage_options = (
        Folder.manage_options[0:3]
        +
        (
            {'label':'Catalog', 'action':'manage_catalog_html'},
            {'label':'Questions', 'action':'manage_questions_html'},
            {'label':'Languages', 'action':'manage_languages_html'},
            {'label':'Mailing system', 'action':'manage_mailing_html'},
        )
        +
        Folder.manage_options[3:]
    )

    _properties = (
        {'id':'title', 'type':'string', 'mode':'w'},
        {'id':'longtitle', 'type':'string', 'mode':'w'},
        {'id':'MetaKeywords', 'type':'lines', 'mode':'w'},
        {'id':'nationalities', 'type':'lines', 'mode':'w'},
        {'id':'servername', 'type':'string', 'mode':'w'},
        {'id':'serverport', 'type':'int', 'mode':'w'},
        {'id':'adminEmail', 'type':'string', 'mode':'w'},
    )

    def __init__(self, id, title, longtitle, servername, serverport, adminEmail, install_base):
        """ BesserWisser constructor"""
        self.id = id
        self.title = title
        self.longtitle = longtitle
        self.servername = servername
        self.serverport = serverport
        self.MetaKeywords = []
        self.nationalities = []
        self.adminEmail = adminEmail
        # create this property to keep the base URL of the BesserWisser 
        # to be inherited by contained objects - to be deleted after testing
        self.install_base = install_base
        CatalogManager.__dict__['__init__'](self)
        Utils.__dict__['__init__'](self)
        LanguagesManager.__dict__['__init__'](self)
        SessionManager.__dict__['__init__'](self)
        BWCookieCrumbler.__dict__['__init__'](self)

    def __setstate__(self, state):
        """ update """
        BesserWisser.inheritedAttribute('__setstate__')(self, state)
        if not hasattr(self, 'longtitle'):
            self.longtitle = ''

    def all_meta_types(self, interfaces=None):
        """ Can only contains BesserWisser objects and some basic stuff """
        y = [
            {'name': Resources.METATYPE_EXPERT_CATEGORY, 'action': 'manage_addCategoryForm'},
        ]
        additional_meta_types = ['DTML Document', 'Image', 'LDAPUserFolder']
        for x in meta_types:
            if x['name'] in additional_meta_types:
                y.append(x)
        return y

    def objectIds(self, spec=None):
        """ Made to hide the acl_users folder from ZMI
            Returns a list of subobject ids of the current object.If 'spec' is specified, 
            returns objects whose meta_type matches 'spec'.
        """
        if spec is not None:
            if type(spec)==type('s'):
                spec=[spec]
            set=[]
            for ob in self._objects:
                if ob['meta_type'] in spec:
                    set.append(ob['id'])
            return set
        return map(lambda i: i['id'], self.filterIds(list(self._objects), 'meta_type', 'User Folder'))

    def filterIds(self, obj_list, attr, value):
        """Filter a list of objects by an attribute value"""
        _alias_operator_getitem = operator.getitem
        len_list = len(obj_list)
        temp = map(None, map(_alias_operator_getitem, obj_list, (attr,)*len_list), (value,)*len_list, obj_list)
        temp = filter(lambda x: x[0]!=x[1], temp)
        return map(_alias_operator_getitem, temp, (-1,)*len(temp))

    ################################################
    #   Generic getters
    ###############################################

    security.declarePublic('getNationalities')
    def getNationalities(self):
        """ Returns the list of nationalities """
        return self.nationalities

    security.declarePublic('getEngine')
    def getEngine(self):
        """ Returns the BesserWisser container """
        return self

    def getAllMainCategories(self):
        """ Returns all main categories """
        return self.objectValues(Resources.METATYPE_EXPERT_CATEGORY)

    def getCategoriesTree(self, category=None, depth=0):
        """ """
        l_categories = []
        if category is None:
            l_objects = self.objectValues(Resources.METATYPE_EXPERT_CATEGORY)
        else:
            l_objects = category.objectValues(Resources.METATYPE_EXPERT_CATEGORY)
        for l_object in l_objects:
            l_categories.append({'object': l_object, 'depth': depth})
            l_categories.extend(self.getCategoriesTree(l_object, depth+1))
        return l_categories

    def getAllQuestionsForExpert(self, ExpertId):
        """ Return all questions for an expert """
        # loop the top categories
        result = []
        for category in self.objectValues(Resources.METATYPE_EXPERT_CATEGORY):
            result.append(category.searchQuestionsOfExpert(ExpertId))
        return result

    def searchAllQuestions(self, p_keyword='', p_category='', p_expert='', p_privacy=0):
        """ Search through all questions """
        return self._searchAllQuestions(p_keyword, p_category, p_expert, p_privacy)

    def latestQuestions(self, nQuestions):
        """ Returns the latest not private nQuestions answered questions """
        return self.utSortObjsListByAttr(self.getCatalogedObjects(Resources.METATYPE_EXPERT_QUESTION), 'answeredDateTime', 1)[:nQuestions]

    security.declarePrivate('LoadINI')
    def LoadINI(self):
        """ Loads values from the .INI file """
        from os.path import join
        l_file = None
        try:
            l_file = open(join(SOFTWARE_HOME, 'Products', 'BesserWisser', 'BesserWisser.ini'), 'r')
        except:
            try:
                l_file = open(join(INSTANCE_HOME, 'Products', 'BesserWisser', 'BesserWisser.ini'), 'r')
            except:
                pass
        if l_file:
            for l_line in l_file.readlines():
                l_line = self.utStrStrip(l_line)
                if l_line != '':
                    l_type, l_values = self.utStrSplit(l_line, '=')
                    # Contruct the default languages as property managed by a separate interface
                    if l_type == 'Language':
                        l_name, l_encoding, l_id = self.utStrSplit(l_values, '|')
                        self.createLanguage(l_id, l_name, l_encoding)
                    # Construct the default nationalities list as regular property
                    if l_type == 'Nationality':
                        self.nationalities.append(l_values)
            l_file.close()
        return 1

    security.declarePublic('emailPassword')
    def emailPassword(self, p_email, REQUEST=None):
        """ Find expert and email password to him/her """
        p_user = ''
        for expert in self.getExpertsFolder().getExperts():
            if expert.Email == p_email:
                p_user = expert.id
                p_pass = self.getExpertUser(p_user).__
                p_to = expert.Email
        if p_user != '':
            ret = self.sendPasswordbyEmail(p_to, self.adminEmail, self.longtitle, p_user, p_pass, self.absolute_url())
            if ret:
                REQUEST.RESPONSE.redirect(self.absolute_url() + '/login_html?mailed=UserFoundMailOK')
            else:
                REQUEST.RESPONSE.redirect(self.absolute_url() + '/login_html?mailed=UserFoundMailFailed')
        else:
            if REQUEST:
                REQUEST.RESPONSE.redirect(self.absolute_url() + '/login_html?mailed=UserNotFound&p_email=' + REQUEST['p_email'])
            else:
                REQUEST.RESPONSE.redirect(self.absolute_url() + '/login_html?mailed=UserNotFound')

    def getExpertUser(self, p_userid):
        """ Returns the User object given the id """
        return self.objectValues('User Folder')[0].getUserById(p_userid)

    security.declarePrivate('__getCategoriesQuestions_forCatalog')
    def __getCategoriesQuestions_forCatalog(self, category=None):
        """ """
        l_categories = []
        if category is None:
            l_objects = self.objectValues(Resources.METATYPE_EXPERT_CATEGORY)
        else:
            l_objects = category.objectValues(Resources.METATYPE_EXPERT_CATEGORY)
        for l_object in l_objects:
            l_categories.append(l_object)
            l_categories.extend(l_object.getCategoryQuestions())
            l_categories.extend(self.__getCategoriesQuestions_forCatalog(l_object))
        return l_categories

    security.declareProtected(view_management_screens, 'manageCatalog')
    def manageCatalog(self, REQUEST=None):
        """Update Catalog"""
        self.clearCatalog()
        #catalog all categories and questions
        for l_item in self.__getCategoriesQuestions_forCatalog():
            self.catalogBesserWisserObject(l_item)
        #catalog all experts
        for l_item in self.getExpertsList():
            self.catalogBesserWisserObject(l_item)
        REQUEST.RESPONSE.redirect('manage_catalog_html')

    security.declareProtected(view_management_screens, 'manageCatalogIndexes')
    def manageCatalogIndexes(self, REQUEST=None):
        """ Reindexing stuff """
        if REQUEST.has_key('reindex'):
            l_ids = self.utConvertToList(REQUEST.get('id', []))
            self.reindexCatalogIndex(l_ids, REQUEST)
        elif REQUEST.has_key('clear'):
            l_ids = self.utConvertToList(REQUEST.get('id', []))
            self.clearCatalogIndex(l_ids)
        elif REQUEST.has_key('add'):
            l_indexname = REQUEST.get('indexname', '')
            l_indextype = REQUEST.get('indextype', '')
            self.createCatalogIndex(l_indexname, l_indextype)
        elif REQUEST.has_key('delete'):
            l_ids = self.utConvertToList(REQUEST.get('id', []))
            self.deleteCatalogIndex(l_ids)
        else:
            pass
        REQUEST.RESPONSE.redirect('manage_catalog_html')

    security.declareProtected(view_management_screens, 'manage_questions_html')
    def manageQuestions(self, REQUEST=None):
        """ Manage Questions objects 
            In this case 'id' reprezents the relative url of the question """
        l_ids = self.utConvertToList(REQUEST.get('ids', []))
        for l_id in l_ids:
            l_question = self.unrestrictedTraverse(l_id)
            l_question.getCategory()._delObject(l_question.id)
        REQUEST.RESPONSE.redirect('manage_questions_html')

    security.declareProtected(view_management_screens, 'manageLanguages')
    def manageLanguages(self, REQUEST=None):
        """ Manage Languages objects """
        if self.utAddObjectAction(REQUEST):
            l_id = REQUEST.get('id', '')
            l_name = REQUEST.get('name', '')
            l_encoding = REQUEST.get('encoding', '')
            self.createLanguage(l_id, l_name, l_encoding)
        elif self.utUpdateObjectAction(REQUEST):
            l_id = REQUEST.get('id', '')
            l_name = REQUEST.get('name', '')
            l_encoding = REQUEST.get('encoding', '')
            self.modifyLanguage(l_id, l_name, l_encoding)
        elif self.utDeleteObjectAction(REQUEST):
            self.deleteLanguage(self.utConvertToList(REQUEST.get('id', [])))
        REQUEST.RESPONSE.redirect('manage_languages_html')

    security.declareProtected(view_management_screens, 'manageMailing')
    def manageMailing(self, ids=[], message='', REQUEST=None):
        """ """
        self.mailToExperts(self.utBuildEmailList(self.utConvertToList(ids)), self.adminEmail, message)
        if REQUEST:
            REQUEST.RESPONSE.redirect('manage_mailing_html')

    def filterExpertsList(self, p_categories=[], p_languages=[]):
        """ Returns a filtered list of experts by category and language """
        l_experts = []
        for l_expert in self.getExpertsList():
            l_flag_categories = 0
            l_flag_languages = 0
            if len(p_categories) > 0:
                l_flag_categories = len(self.utListIntersection(l_expert.Expertise, p_categories)) > 0
            else:
                l_flag_categories = 1
            if len(p_languages) > 0:
                l_flag_languages = len(self.utListIntersection(l_expert.LanguagesUnderstood, p_languages)) > 0
            else:
                l_flag_languages = 1
            if l_flag_categories and l_flag_languages:
                l_experts.append(l_expert)
        return l_experts


    security.declarePublic('amIanExpert')
    def amIanExpert(self):
        """ True if the user logged in has an Expert role, false otherwise """
        return self.getEnroledExpert(self.REQUEST.AUTHENTICATED_USER.getUserName()) is not None

    security.declarePublic('getExpertsFolder')
    def getExpertsFolder(self):
        """ Returns the current Experts Folder """
        return self.unrestrictedTraverse(self.absolute_url(1) + '/Experts')

    security.declarePublic('getExpertsList')
    def getExpertsList(self):
        """ Returns the current Experts Folder content """
        try:
            return self.getExpertsFolder().getExperts()
        except:
            return []

    def getEnroledExpert(self, expert_id):
        """ Returns the Expert object from Experts folder """
        try:
            return self.getExpertsFolder()._getOb(expert_id)
        except:
            return None

    def getEnroledExpertsObjects(self, experts_ids):
        """ """
        get_expert = self.getExpertsFolder()._getOb
        res = []
        for id in experts_ids:
            try:
                res.append(get_expert(id))
            except:
                pass
        return res

    def catalogBesserWisserObject(self, obj):
        """ Catalogs the contained objects 
            Available through aquisition to all subobjects """
        try:
            self.catalogObject(obj)
        except:
            pass

    def uncatalogBesserWisserObject(self, obj):
        """ Uncatalogs the contained objects 
            Available through aquisition to all subobjects """
        try:
            self.uncatalogObject(obj)
        except:
            pass

    def recatalogBesserWisserObject(self, p_ob):
        """ Re-Catalog an object """
        try:
            self.recatalogObject(p_ob)
        except:
            pass

    def make_Id(self, theId):
        """ Removes not allowed chars from an id - characters '/ +@-,' translated to '_'
            Used to create a valid id from an user input. """
        transtab = string.maketrans('/\ +@-,','_______')
        newid = string.translate(theId, transtab,'?&!;()<=>*#[]{}^~:|\/$%')
        return newid


    ##############################################
    # Tables records sort methods
    ##############################################
    security.declarePrivate('getFilterString')
    def getFilterString(self, **args):
        """ Returns a string like: property=value&property=value ... """
        filterquerystring = ''
        for arg in args.keys():
            if args[arg] != '':
                filterquerystring += arg + '='+ args[arg] + '&'
        if filterquerystring != '':
            filterquerystring = filterquerystring[0:len(filterquerystring)-1]
        return filterquerystring

    security.declarePrivate('getSortQueryString')
    def getSortQueryString(self, property, sortby, how, filterquerystring):
        """ Returns a string like : sort=property&how=(asc|desc) """
        querystring=''
        if property==sortby:
            if how == 'asc':
                querystring += 'sort=' + property + '&how=desc'
            else:
                querystring += 'sort=' + property + '&how=asc'
        else:
            querystring += 'sort=' + property + '&how=asc'
        if filterquerystring != '':
            querystring += '&' + filterquerystring
        return querystring

    security.declarePrivate('getSortPicture')
    def getSortPicture(self, property, sortby, how):
        """ Returns the name of the sort picture """
        picture='spacer.gif'
        if property==sortby:
            if (how == 'asc'):
                picture = 'sort_asc.gif'
            elif (how == 'desc'):
                picture = 'sort_desc.gif'
        return picture

    security.declarePrivate('getStartStopTotalValues')
    def getStartStopTotalValues(self, issues, REQUEST):
        """ get the start, stop , total values for paging a list of questions"""
        #initialize
        l_start = REQUEST.get('start', 1)
        l_stop = 0
        l_total = len(issues)
        #compute
        try:
            l_start = int(l_start)
        except:
            l_start = 1
        l_start = min(l_start, l_total)
        l_stop = min(l_start + 10 - 1, l_total)
        return (l_start, l_stop, l_total)

    security.declarePrivate('getSortFilteredQuestions')
    def getSortFilteredQuestions(self, p_expertId, p_isPrivate, p_sort_by, p_sort_how):
        """ sort/filter questions """
        if p_sort_how == 'desc':
            p_sort_how = 1
        else:
            p_sort_how = 0
        return self.utSortObjsListByAttr(self.getFilteredQuestions(p_expertId, p_isPrivate), p_sort_by, p_sort_how)


    ##############################################
    # Generic forms
    ##############################################

    security.declareProtected(view, 'besserwisser_css')
    besserwisser_css = Globals.DTMLFile('dtml/BesserWisser_css', globals())

    security.declareProtected(view, 'besserwisser_html_header')
    besserwisser_html_header = Globals.DTMLFile('dtml/BesserWisser_html_header', globals())

    security.declareProtected(view, 'besserwisser_html_footer')
    besserwisser_html_footer = Globals.DTMLFile('dtml/BesserWisser_html_footer', globals())

    security.declareProtected(view, 'index_html')
    index_html = Globals.DTMLFile('dtml/BesserWisser_index', globals())

    security.declareProtected(view, 'search_html')
    search_html = Globals.DTMLFile('dtml/BesserWisser_search', globals())

    security.declareProtected(view, 'termsforexperts')
    termsforexperts = Globals.DTMLFile('dtml/BesserWisser_termsforexperts',globals())

    security.declareProtected(view, 'termsofuse')
    termsofuse = Globals.DTMLFile('dtml/BesserWisser_termsofuse',globals())

    security.declareProtected(view, 'login_html')
    login_html = Globals.DTMLFile('dtml/BesserWisser_login',globals())

    security.declareProtected(view, 'logout_html')
    logout_html = Globals.DTMLFile('dtml/BesserWisser_logout',globals())

    security.declareProtected(view, 'rating_html')
    rating_html = Globals.DTMLFile('dtml/BesserWisser_rating',globals())

    security.declareProtected(view, 'help_html')
    help_html = Globals.DTMLFile('dtml/BesserWisser_help',globals())

    security.declarePrivate('errorsbox_html')
    errorsbox_html = Globals.DTMLFile('dtml/BesserWisser_errorsbox', globals())

    security.declarePrivate('tooltip_js')
    tooltip_js = Globals.DTMLFile('dtml/tooltip_js', globals())

    security.declareProtected(view, 'optiontabs_start')
    optiontabs_start = Globals.DTMLFile('dtml/BesserWisser_optiontabs_start',globals())

    security.declareProtected(view, 'optiontabs_end')
    optiontabs_end = Globals.DTMLFile('dtml/BesserWisser_optiontabs_end',globals())

    security.declareProtected(view, 'latestquestions_rdf')
    latestquestions_rdf = Globals.DTMLFile('dtml/BesserWisser_latestquestions_rdf',globals())

    ##############################################
    # Administrative functions
    ##############################################

    security.declareProtected(view_management_screens, 'manage_languages_html')
    manage_languages_html = Globals.DTMLFile('dtml/BesserWisser_manage_languages', globals())

    security.declareProtected(view_management_screens, 'manage_catalog_html')
    manage_catalog_html = Globals.DTMLFile('dtml/BesserWisser_manage_catalog', globals())

    security.declareProtected(view_management_screens, 'manage_catalog_object_html')
    manage_catalog_object_html = Globals.DTMLFile('dtml/BesserWisser_manage_catalog_object', globals())

    security.declareProtected(view_management_screens, 'manage_questions_html')
    manage_questions_html = Globals.DTMLFile('dtml/BesserWisser_manage_questions', globals())

    security.declareProtected(view_management_screens, 'manage_mailing_html')
    manage_mailing_html = Globals.DTMLFile('dtml/BesserWisser_manage_mailing', globals())

    security.declareProtected(view, 'manage_addExpertsFolder')
    manage_addExpertsFolder = ExpertsFolder.manage_addExpertsFolder

    security.declareProtected(view, 'manage_addExpertsFolderForm')
    manage_addExpertsFolderForm = ExpertsFolder.manage_addExpertsFolderForm

    security.declareProtected(Resources.PERMISSION_ADD_EXPERT_CATEGORY, 'manage_addCategoryForm')
    manage_addCategoryForm = Category.manage_addCategoryForm

    security.declareProtected(Resources.PERMISSION_ADD_EXPERT_CATEGORY, 'manage_addCategory')
    manage_addCategory = Category.manage_addCategory


    #################################
    #   IMPORT ENVIRO EXPERTS
    #################################
    security.declareProtected(view_management_screens, 'importCategories')
    def importCategories(self):
        """ import Categories from EnviroExperts """
        import xmlrpclib
        server = xmlrpclib.Server('http://experts.ewindows.eu.org/')
        l_container_url, l_categories_list = server.enviro.exportCategories()
        for l_category in l_categories_list:
            l_category_path = l_category[0].replace(l_container_url + '/', '')
            l_category_parent_path = l_category[1].replace(l_container_url + '/', '')
            if l_category_parent_path == l_container_url:
                #main category
                self.manage_addCategory(l_category_path, l_category[2].replace('\n', ' '), l_category[3].replace('\n', ' '), map(self.make_Id, self.utConvertToList(l_category[4])), keywords=l_category[5], CId=l_category[6], creationDate=l_category[7])
            else:
                #subcategory
                l_parent_category = self.unrestrictedTraverse(self.absolute_url(1) + '/' + l_category_parent_path)
                l_parent_category.manage_addCategory(l_category_path.replace(l_category_parent_path + '/', ''), l_category[2].replace('\n', ' '), l_category[3].replace('\n', ' '), l_category[4], keywords=l_category[5], CId=l_category[6], creationDate=l_category[7])
        return "Import OK"

    security.declarePrivate('importExpert')
    def importExpert(self, expert_folder, name='', password='', Firstname='', Lastname='', Nationality='', Email='',
            ProfileEnglish='', ProfileNative='', Employer='', Expertise=[], LanguagesUnderstood=[],
            QuestionsPerDay=0, ProfileNativeIsIn='', isOrganisation=0,
            onVacation=0, ExpectedReturnDate=None):
        """ Import an expert """
        ob = Expert.Expert(name, password, Firstname.encode('latin-1'), Lastname.encode('latin-1'), Email, ProfileNativeIsIn,
                LanguagesUnderstood, Nationality, ProfileEnglish, ProfileNative, Employer,
                Expertise, QuestionsPerDay, isOrganisation, '', onVacation, ExpectedReturnDate)
        expert_folder._setObject(name, ob)
        ob = expert_folder._getOb(name)
        ob.updateExpertExpertise([], Expertise)
        self.acl_users._addRole(Expert._EXPERT_ROLE)
        self.manage_role(Expert._EXPERT_ROLE, permissions=['Access contents information', view])
        self.acl_users._addUser(name, password, password, ['Manager'],0)

    security.declareProtected(view_management_screens, 'importExperts')
    def importExperts(self):
        """ Import Experts from EnviroExperts """
        import xmlrpclib
        server = xmlrpclib.Server('http://experts.ewindows.eu.org/')
        l_experts_list = server.enviro.exportExperts()
        l_expert_folder = self.getExpertsFolder()
        #get a dictionary with languages, label:id
        l_languages = self.getLanguagesNamesToIds()
        for l_expert in l_experts_list:
            #convert Expertise: CId -> categories absolute_urls.
            l_expert_Expertise = self.utConvertToList(l_expert[9])
            l_expert_Expertise_new = []
            for l_item in l_expert_Expertise:
                l_category = self.getCategoryByCId(l_item)
                if l_category:
                    l_expert_Expertise_new.append(l_category.absolute_url(1))
                else:
                    #print "Expertise error for expert %s with category CId %s" % (l_expert[0], l_item)
                    pass
            #convert languages labels to ids
            l_expert_LanguagesUnderstood = l_expert[10]
            l_expert_LanguagesUnderstood_new = []
            for l_item in l_expert_LanguagesUnderstood:
                if l_languages.has_key(l_item):
                    l_expert_LanguagesUnderstood_new.append(l_languages[l_item])
            #convert ProfileNativeIsIn from 'label|encoding|id' to id
            l_expert_ProfileNativeIsIn = l_expert[12].split('|')
            if len(l_expert_ProfileNativeIsIn) == 3:
                l_expert_ProfileNativeIsIn_new = l_expert_ProfileNativeIsIn[2]
            else:
                l_expert_ProfileNativeIsIn_new = ''
            #add expert
            print self.make_Id(l_expert[0])
            self.importExpert(l_expert_folder, self.make_Id(l_expert[0]), l_expert[1], l_expert[2], l_expert[3],
                l_expert[4], l_expert[5], l_expert[6], l_expert[7], l_expert[8], l_expert_Expertise_new,
                l_expert_LanguagesUnderstood_new, l_expert[11], l_expert_ProfileNativeIsIn_new, l_expert[13], l_expert[14], l_expert[15])
        return "Import OK"

    security.declarePrivate('importExpert')
    def imporQuestion(self, question_category, id='', title='', question='', Language='', InquirersName='',
            InquirersEmail='', ExpertId='', isPrivate=0, Answer='', Rating='', Timelineness='',
            askedDateTime=None, answeredDateTime=None):
        """ Import an expert """
        access_code = self.utGenRandomKey()
        try: askedDateTime = DateTime(askedDateTime)
        except: askedDateTime = None
        try: answeredDateTime = DateTime(answeredDateTime)
        except: answeredDateTime = None
        ob = Question.ExpertQuestion(id, title, question, Language, InquirersName, InquirersEmail,
                ExpertId, isPrivate, Answer, Rating, Timelineness, askedDateTime, answeredDateTime,
                access_code)
        question_category._setObject(id, ob)

    security.declareProtected(view_management_screens, 'importQuestions')
    def importQuestions(self):
        """ Import Questions from EnviroExperts """
        import xmlrpclib
        server = xmlrpclib.Server('http://experts.ewindows.eu.org/')
        l_questions_list = server.enviro.exportQuestions()
        #get a dictionary with languages, label:id
        l_languages = self.getLanguagesNamesToIds()
        for l_question in l_questions_list:
            l_category = self.getCategoryByCId(l_question[1])
            if l_category:
                #convert languages label to id
                l_question_Language = l_question[4]
                if l_languages.has_key(l_question_Language):
                    l_question_Language_new = l_languages[l_question_Language]
                else:
                    l_question_Language_new = ''
                #print "add question ", l_question[0], l_category.absolute_url(1)
                self.imporQuestion(l_category, self.make_Id(l_question[0]), l_question[2].encode('latin-1'), l_question[3].encode('latin-1'), l_question_Language_new,
                    l_question[5].encode('latin-1'), l_question[6], self.make_Id(l_question[7]), l_question[8], l_question[9].encode('latin-1'), l_question[10],
                    l_question[11], l_question[12], l_question[13])
            else:
                #print "cannot import question %s for category %s\n" % (l_question[0], l_question[1])
                pass
        return "Import OK"

    security.declareProtected(view_management_screens, 'importQuestions')
    def cleanUpAndFixes(self):
        """ Clean up invalid expert ids """
        for l_category in self.getCatalogedObjects(Resources.METATYPE_EXPERT_CATEGORY):
            l_buf_enroledExperts = map(lambda x: x.id, self.getEnroledExpertsObjects(l_category.enroledExperts))
            l_dict = {}
            for l_item in l_buf_enroledExperts:
                l_dict[l_item] = 1
            l_category.enroledExperts = l_dict.keys()
            self.catalogBesserWisserObject(l_category)
        return "CLEAN UP AND FIXES OK"

Globals.InitializeClass(BesserWisser)
